@@ -5,7 +5,7 @@ import logging
 from qtpy.QtWidgets import (QWidget, QStyle, QStyleOption)
 from qtpy.QtGui import (QColor, QPainter, QBrush, QPen, QPolygon, QPolygonF, QPixmap,
                             QMovie)
-from qtpy.QtCore import Property, Qt, QPoint, QPointF, QSize, Slot, QTimer
+from qtpy.QtCore import Property, Qt, QPoint, QPointF, QSize, Slot, QTimer, QRectF
 from qtpy.QtDesigner import QDesignerFormWindowInterface
 from .base import PyDMWidget
 from ..utilities import is_qt_designer, find_file
@@ -66,7 +66,7 @@ class PyDMDrawing(QWidget, PyDMWidget):
     def __init__(self, parent=None, init_channel=None):
         self._rotation = 0.0
         self._brush = QBrush(Qt.SolidPattern)
-        self._original_brush = None
+        self._original_brush = self._brush
         self._painter = QPainter()
         self._pen = QPen(Qt.NoPen)
         self._pen_style = Qt.NoPen
@@ -406,7 +406,7 @@ class PyDMDrawing(QWidget, PyDMWidget):
             return
         if new_width != self._pen_width:
             self._pen_width = new_width
-            self._pen.setWidth(self._pen_width)
+            self._pen.setWidthF(float(self._pen_width))
             self.update()
 
     @Property(float)
@@ -458,12 +458,41 @@ class PyDMDrawingLine(PyDMDrawing):
     init_channel : str, optional
         The channel to be used by the widget.
     """
+
     def __init__(self, parent=None, init_channel=None):
         super(PyDMDrawingLine, self).__init__(parent, init_channel)
-        self.rotation = 45
+        self._arrow_end_point_selection = False
+        self._arrow_start_point_selection = False
+        self.rotation = 0
         self.penStyle = Qt.SolidLine
         self.penWidth = 1
 
+    @staticmethod
+    def _arrow_points(startpoint, endpoint, height, width):
+        """
+        Returns the three points needed to make a triangle with .drawPolygon
+        """
+        diff_x = startpoint.x() - endpoint.x()
+        diff_y = startpoint.y() - endpoint.y()
+
+        length = math.sqrt(diff_x ** 2 + diff_y ** 2)
+
+        norm_x = diff_x / length
+        norm_y = diff_y / length
+
+        perp_x = -norm_y
+        perp_y = norm_x
+
+        left_x = endpoint.x() + height * norm_x + width * perp_x
+        left_y = endpoint.y() + height * norm_y + width * perp_y
+        right_x = endpoint.x() + height * norm_x - width * perp_x
+        right_y = endpoint.y() + height * norm_y - width * perp_y
+
+        left = QPointF(left_x, left_y)
+        right = QPointF(right_x, right_y)
+
+        return QPolygonF([left, endpoint, right])
+
     def draw_item(self, painter):
         """
         Draws the line after setting up the canvas with a call to
@@ -471,8 +500,66 @@ class PyDMDrawingLine(PyDMDrawing):
         """
         super(PyDMDrawingLine, self).draw_item(painter)
         x, y, w, h = self.get_bounds()
-        painter.drawLine(x, y, w, h)
 
+        start_point = QPointF(x, 0)
+        end_point = QPointF(x + w, 0)
+
+        painter.drawLine(start_point, end_point)
+        if self._arrow_end_point_selection:
+            points = self._arrow_points(start_point, end_point, 6, 6)
+            painter.drawPolygon(points)
+
+        if self._arrow_start_point_selection:
+            points = self._arrow_points(end_point, start_point, 6, 6)
+            painter.drawPolygon(points)
+
+    @Property(bool)
+    def arrowEndPoint(self):
+        """
+        If True, an arrow will be drawn at the end of the line.
+
+        Returns
+        -------
+        bool
+        """
+        return self._arrow_end_point_selection
+
+    @arrowEndPoint.setter
+    def arrowEndPoint(self, new_selection):
+        """
+        If True, an arrow will be drawn at the end of the line.
+
+        Parameters
+        -------
+        new_selection : bool
+        """
+        if self._arrow_end_point_selection != new_selection:
+            self._arrow_end_point_selection = new_selection
+            self.update()
+
+    @Property(bool)
+    def arrowStartPoint(self):
+        """
+        If True, an arrow will be drawn at the start of the line.
+
+        Returns
+        -------
+        bool
+        """
+        return self._arrow_start_point_selection
+
+    @arrowStartPoint.setter
+    def arrowStartPoint(self, new_selection):
+        """
+        If True, an arrow will be drawn at the start of the line.
+
+        Parameters
+        -------
+        new_selection : bool
+        """
+        if self._arrow_start_point_selection != new_selection:
+            self._arrow_start_point_selection = new_selection
+            self.update()
 
 class PyDMDrawingImage(PyDMDrawing):
     """
@@ -642,7 +729,7 @@ class PyDMDrawingImage(PyDMDrawing):
         super(PyDMDrawingImage, self).draw_item(painter)
         x, y, w, h = self.get_bounds(maxsize=True, force_no_pen=True)
         if not isinstance(self._pixmap, QMovie):
-            _scaled = self._pixmap.scaled(w, h, self._aspect_ratio_mode,
+            _scaled = self._pixmap.scaled(int(w), int(h), self._aspect_ratio_mode,
                                           Qt.SmoothTransformation)
             # Make sure the image is centered if smaller than the widget itself
             if w > _scaled.width():
@@ -651,7 +738,7 @@ class PyDMDrawingImage(PyDMDrawing):
             if h > _scaled.height():
                 logger.debug("Centering image vertically ...")
                 y += (h - _scaled.height())/2
-            painter.drawPixmap(x, y, _scaled)
+            painter.drawPixmap(QPointF(x, y), _scaled)
 
     def movie_frame_changed(self, frame_no):
         """
@@ -709,7 +796,7 @@ class PyDMDrawingRectangle(PyDMDrawing):
         """
         super(PyDMDrawingRectangle, self).draw_item(painter)
         x, y, w, h = self.get_bounds(maxsize=True)
-        painter.drawRect(x, y, w, h)
+        painter.drawRect(QRectF(x, y, w, h))
 
 
 class PyDMDrawingTriangle(PyDMDrawing):
@@ -729,9 +816,9 @@ class PyDMDrawingTriangle(PyDMDrawing):
 
     def _calculate_drawing_points(self, x, y, w, h):
         return [
-            QPoint(x, h / 2.0),
-            QPoint(x, y),
-            QPoint(w / 2.0, y)
+            QPointF(x, h / 2.0),
+            QPointF(x, y),
+            QPointF(w / 2.0, y)
         ]
 
     def draw_item(self, painter):
@@ -743,7 +830,7 @@ class PyDMDrawingTriangle(PyDMDrawing):
         x, y, w, h = self.get_bounds(maxsize=True)
         points = self._calculate_drawing_points(x, y, w, h)
 
-        painter.drawPolygon(QPolygon(points))
+        painter.drawPolygon(QPolygonF(points))
 
 
 class PyDMDrawingEllipse(PyDMDrawing):
@@ -880,7 +967,7 @@ class PyDMDrawingArc(PyDMDrawing):
         super(PyDMDrawingArc, self).draw_item(painter)
         maxsize = not self.is_square()
         x, y, w, h = self.get_bounds(maxsize=maxsize)
-        painter.drawArc(x, y, w, h, self._start_angle, self._span_angle)
+        painter.drawArc(QRectF(x, y, w, h), int(self._start_angle), int(self._span_angle))
 
 
 class PyDMDrawingPie(PyDMDrawingArc):
@@ -906,7 +993,7 @@ class PyDMDrawingPie(PyDMDrawingArc):
         super(PyDMDrawingPie, self).draw_item(painter)
         maxsize = not self.is_square()
         x, y, w, h = self.get_bounds(maxsize=maxsize)
-        painter.drawPie(x, y, w, h, self._start_angle, self._span_angle)
+        painter.drawPie(QRectF(x, y, w, h), int(self._start_angle), int(self._span_angle))
 
 
 class PyDMDrawingChord(PyDMDrawingArc):
@@ -932,7 +1019,7 @@ class PyDMDrawingChord(PyDMDrawingArc):
         super(PyDMDrawingChord, self).draw_item(painter)
         maxsize = not self.is_square()
         x, y, w, h = self.get_bounds(maxsize=maxsize)
-        painter.drawChord(x, y, w, h, self._start_angle, self._span_angle)
+        painter.drawChord(QRectF(x, y, w, h), int(self._start_angle), int(self._span_angle))
 
 
 class PyDMDrawingPolygon(PyDMDrawing):
@@ -1025,7 +1112,7 @@ class PyDMDrawingPolyline(PyDMDrawing):
             # drawing coordinates are centered: (0,0) is in center
             # our points are absolute: (0,0) is upper-left corner
             u, v = map(int, pt.split(","))
-            return QPoint(u+x, v+y)
+            return QPointF(u+x, v+y)
 
         if len(self._points) > 1:
             for i, p1 in enumerate(self._points[:-1]):
